#include <iostream>
#include "MyClass.h"
#include "TRandom3.h"
#include "TFile.h"

int main ( int argc, char **argv )
{
    int imax = 1000000;

    std::cout << argc << " arguments have been provided" << std::endl;

    for ( int i = 0; i < argc; i++ )
    {
        std::cout << "Argument #" << i << ": " << argv[i] << std::endl;
    }

    if ( argc == 3 )
    {
        if ( strcmp ( argv[1], "-nevents" ) == 0 )
        {
            imax = std::stoi ( argv[2] );
        }
        else
        {
            std::cerr << "Invalid arguments. Leave it blank or type \"-nevents XXX\"" << std::endl;

            return -1;
        }
    }
    else if ( argc != 1 )
    {
        std::cerr << "Invalid arguments. Leave it blank or type \"-nevents XXX\"" << std::endl;

        return -1;
    }

    TRandom3* rng = new TRandom3();

    TTree* tree = new TTree ( "return_vector_issue", "return_vector_issue" );

    MyClass *myCl = new MyClass();
    std::vector<MyClass> *myClVect = new std::vector<MyClass>;

    myClVect->clear();

    tree->Branch ( "MyClass", &myCl );
    tree->Branch ( "MyClassVect", &myClVect );

    TTree* tree2 = new TTree ( "direct_vector_branch", "direct_vector_branch" );

    std::vector<float> vec1;
    std::vector<float> vec2;
    std::vector<float> vec3;
    std::vector<float> vec4;
    std::vector<float> vec5;
    std::vector<float> vec6;

    tree2->Branch ( "vec1", &vec1 );
    tree2->Branch ( "vec2", &vec2 );
    tree2->Branch ( "vec3", &vec3 );
    tree2->Branch ( "vec4", &vec4 );
    tree2->Branch ( "vec5", &vec5 );
    tree2->Branch ( "vec6", &vec6 );

    TTree* tree3 = new TTree ( "nested_vector_branch", "nested_vector_branch" );

    MyBaseClass* myBase = new MyBaseClass();

    tree3->Branch ( "myBase", &myBase );

    for ( int i = 0; i < imax; i++ )
    {
        if ( i%1000 == 0 ) std::cout << "Generated " << i << " out of " << imax << " events\r" << std::flush;

        // ---------------- return vector issue part ----------------------- //

        myCl->Clear();
        myClVect->clear();

        myCl->SetSimpleMember ( ( unsigned short ) rng->Uniform ( 0,99 ) );

        for ( int j =0; j < ( unsigned short ) rng->Uniform ( 3,8 ); j++ )
        {
            myCl->AddValToVect ( ( unsigned short ) rng->Uniform ( 0,99 ) );
        }

        for ( int j =0; j < ( unsigned short ) rng->Uniform ( 1,4 ); j++ )
        {
            MyClass newMyCl;

            newMyCl.Clear();

            newMyCl.SetSimpleMember ( ( unsigned short ) rng->Uniform ( 0,99 ) );

            for ( int k =0; k < ( unsigned short ) rng->Uniform ( 3,8 ); k++ )
            {
                newMyCl.AddValToVect ( ( unsigned short ) rng->Uniform ( 0,99 ) );
            }

            myClVect->push_back ( newMyCl );

            // ---------------- file size with custom classes part ----------------------- //

            vec1.clear();
            vec2.clear();
            vec3.clear();
            vec4.clear();
            vec5.clear();
            vec6.clear();

            myBase->Clear();

            for ( int j =0; j < rng->Uniform ( 3,8 ); j++ )
            {
                float rdm1 = rng->Uniform ( 0,99 ) ;
                float rdm2 = rng->Uniform ( 0,99 ) ;

                vec1.push_back ( rdm1 );
                vec2.push_back ( rdm2 );


                myBase->myCont.myCont1.myVect1.push_back ( rdm1 );
                myBase->myCont.myCont1.myVect2.push_back ( rdm2 );
            }

            for ( int j =0; j < rng->Uniform ( 3,8 ); j++ )
            {
                float rdm1 = rng->Uniform ( 0,99 ) ;
                float rdm2 = rng->Uniform ( 0,99 ) ;

                vec3.push_back ( rdm1 );
                vec4.push_back ( rdm2 );


                myBase->myCont.myCont2.myVect1.push_back ( rdm1 );
                myBase->myCont.myCont2.myVect2.push_back ( rdm2 );
            }

            for ( int j =0; j < rng->Uniform ( 3,8 ); j++ )
            {
                float rdm1 = rng->Uniform ( 0,99 ) ;
                float rdm2 = rng->Uniform ( 0,99 ) ;

                vec5.push_back ( rdm1 );
                vec6.push_back ( rdm2 );


                myBase->myCont.myCont3.myVect1.push_back ( rdm1 );
                myBase->myCont.myCont3.myVect2.push_back ( rdm2 );
            }
        }

        tree->Fill();
        tree2->Fill();
        tree3->Fill();
    }

    std::cout << std::endl;

    TFile *file = new TFile ( "./return_vector_issue.root", "recreate" );

    file->cd();

    tree->Write();

    file->Close();

    file = new TFile ( "./direct_vector_branches.root", "recreate" );

    file->cd();

    tree2->Write();

    file->Close();

    file = new TFile ( "./nested_vector_branches.root", "recreate" );

    file->cd();

    tree3->Write();

    file->Close();

    return 1;
}
